@@ -34,7 +34,7 @@ const char *fz_stext_options_usage =
 	"\n";
 
 fz_stext_page *
-fz_new_stext_page(fz_context *ctx, const fz_rect *mediabox)
+fz_new_stext_page(fz_context *ctx, fz_rect mediabox)
 {
 	fz_pool *pool = fz_new_pool(ctx);
 	fz_stext_page *page = NULL;
@@ -42,7 +42,7 @@ fz_new_stext_page(fz_context *ctx, const fz_rect *mediabox)
 	{
 		page = fz_pool_alloc(ctx, pool, sizeof(*page));
 		page->pool = pool;
-		page->mediabox = *mediabox;
+		page->mediabox = mediabox;
 		page->first_block = NULL;
 		page->last_block = NULL;
 	}
@@ -91,17 +91,13 @@ add_text_block_to_page(fz_context *ctx, fz_stext_page *page)
 }
 
 static fz_stext_block *
-add_image_block_to_page(fz_context *ctx, fz_stext_page *page, const fz_matrix *ctm, fz_image *image)
+add_image_block_to_page(fz_context *ctx, fz_stext_page *page, fz_matrix ctm, fz_image *image)
 {
 	fz_stext_block *block = add_block_to_page(ctx, page);
 	block->type = FZ_STEXT_BLOCK_IMAGE;
-	block->u.i.transform = *ctm;
+	block->u.i.transform = ctm;
 	block->u.i.image = fz_keep_image(ctx, image);
-	block->bbox.x0 = 0;
-	block->bbox.y0 = 0;
-	block->bbox.x1 = 1;
-	block->bbox.y1 = 1;
-	fz_transform_rect(&block->bbox, ctm);
+	block->bbox = fz_transform_rect(fz_unit_rect, ctm);
 	return block;
 }
 
@@ -124,18 +120,8 @@ add_line_to_block(fz_context *ctx, fz_stext_page *page, fz_stext_block *block, c
 	return line;
 }
 
-static float min4(float a, float b, float c, float d)
-{
-	return fz_min(fz_min(a, b), fz_min(c, d));
-}
-
-static float max4(float a, float b, float c, float d)
-{
-	return fz_max(fz_max(a, b), fz_max(c, d));
-}
-
 static fz_stext_char *
-add_char_to_line(fz_context *ctx, fz_stext_page *page, fz_stext_line *line, const fz_matrix *trm, fz_font *font, float size, int c, fz_point *p, fz_point *q)
+add_char_to_line(fz_context *ctx, fz_stext_page *page, fz_stext_line *line, fz_matrix trm, fz_font *font, float size, int c, fz_point *p, fz_point *q)
 {
 	fz_stext_char *ch = fz_pool_alloc(ctx, page->pool, sizeof *line->first_char);
 	fz_point a, d;
@@ -162,19 +148,19 @@ add_char_to_line(fz_context *ctx, fz_stext_page *page, fz_stext_line *line, cons
 	}
 	else
 	{
-		fz_rect *bbox = fz_font_bbox(ctx, font);
-		a.x = bbox->x1;
-		d.x = bbox->x0;
+		fz_rect bbox = fz_font_bbox(ctx, font);
+		a.x = bbox.x1;
+		d.x = bbox.x0;
 		a.y = 0;
 		d.y = 0;
 	}
-	fz_transform_vector(&a, trm);
-	fz_transform_vector(&d, trm);
+	a = fz_transform_vector(a, trm);
+	d = fz_transform_vector(d, trm);
 
-	ch->bbox.x0 = min4(p->x + a.x, q->x + a.x, p->x + d.x, q->x + d.x);
-	ch->bbox.x1 = max4(p->x + a.x, q->x + a.x, p->x + d.x, q->x + d.x);
-	ch->bbox.y0 = min4(p->y + a.y, q->y + a.y, p->y + d.y, q->y + d.y);
-	ch->bbox.y1 = max4(p->y + a.y, q->y + a.y, p->y + d.y, q->y + d.y);
+	ch->quad.ll = fz_make_point(p->x + d.x, p->y + d.y);
+	ch->quad.ul = fz_make_point(p->x + a.x, p->y + a.y);
+	ch->quad.lr = fz_make_point(q->x + d.x, q->y + d.y);
+	ch->quad.ur = fz_make_point(q->x + a.x, q->y + a.y);
 
 	return ch;
 }
@@ -212,14 +198,14 @@ direction_from_bidi_class(int bidiclass, int curdir)
 	}
 }
 
-static int
+static float
 vec_dot(const fz_point *a, const fz_point *b)
 {
 	return a->x * b->x + a->y * b->y;
 }
 
 static void
-fz_add_stext_char_imp(fz_context *ctx, fz_stext_device *dev, fz_font *font, int c, int glyph, fz_matrix *trm, float adv, int wmode)
+fz_add_stext_char_imp(fz_context *ctx, fz_stext_device *dev, fz_font *font, int c, int glyph, fz_matrix trm, float adv, int wmode)
 {
 	fz_stext_page *page = dev->page;
 	fz_stext_block *cur_block;
@@ -248,9 +234,8 @@ fz_add_stext_char_imp(fz_context *ctx, fz_stext_device *dev, fz_font *font, int
 		dir.x = 0;
 		dir.y = -1;
 	}
-	fz_transform_vector(&dir, trm);
-	ndir = dir;
-	fz_normalize_vector(&ndir);
+	dir = fz_transform_vector(dir, trm);
+	ndir = fz_normalize_vector(dir);
 
 	size = fz_matrix_expansion(trm);
 
@@ -271,17 +256,17 @@ fz_add_stext_char_imp(fz_context *ctx, fz_stext_device *dev, fz_font *font, int
 	 */
 	if (wmode == 0)
 	{
-		p.x = trm->e;
-		p.y = trm->f;
-		q.x = trm->e + adv * dir.x;
-		q.y = trm->f + adv * dir.y;
+		p.x = trm.e;
+		p.y = trm.f;
+		q.x = trm.e + adv * dir.x;
+		q.y = trm.f + adv * dir.y;
 	}
 	else
 	{
-		p.x = trm->e - adv * dir.x;
-		p.y = trm->f - adv * dir.y;
-		q.x = trm->e;
-		q.y = trm->f;
+		p.x = trm.e - adv * dir.x;
+		p.y = trm.f - adv * dir.y;
+		q.x = trm.e;
+		q.y = trm.f;
 	}
 
 	/* Find current position to enter new text. */
@@ -401,11 +386,11 @@ fz_add_stext_char_imp(fz_context *ctx, fz_stext_device *dev, fz_font *font, int
 	dev->pen = q;
 
 	dev->new_obj = 0;
-	dev->trm = *trm;
+	dev->trm = trm;
 }
 
 static void
-fz_add_stext_char(fz_context *ctx, fz_stext_device *dev, fz_font *font, int c, int glyph, fz_matrix *trm, float adv, int wmode)
+fz_add_stext_char(fz_context *ctx, fz_stext_device *dev, fz_font *font, int c, int glyph, fz_matrix trm, float adv, int wmode)
 {
 	/* ignore when one unicode character maps to multiple glyphs */
 	if (c == -1)
@@ -476,7 +461,7 @@ fz_add_stext_char(fz_context *ctx, fz_stext_device *dev, fz_font *font, int c, i
 }
 
 static void
-fz_stext_extract(fz_context *ctx, fz_stext_device *dev, fz_text_span *span, const fz_matrix *ctm)
+fz_stext_extract(fz_context *ctx, fz_stext_device *dev, fz_text_span *span, fz_matrix ctm)
 {
 	fz_font *font = span->font;
 	fz_matrix tm = span->trm;
@@ -489,14 +474,14 @@ fz_stext_extract(fz_context *ctx, fz_stext_device *dev, fz_text_span *span, cons
 
 	tm.e = 0;
 	tm.f = 0;
-	fz_concat(&trm, &tm, ctm);
+	trm = fz_concat(tm, ctm);
 
 	for (i = 0; i < span->len; i++)
 	{
 		/* Calculate new pen location and delta */
 		tm.e = span->items[i].x;
 		tm.f = span->items[i].y;
-		fz_concat(&trm, &tm, ctm);
+		trm = fz_concat(tm, ctm);
 
 		/* Calculate bounding box and new pen position based on font metrics */
 		if (span->items[i].gid >= 0)
@@ -504,12 +489,12 @@ fz_stext_extract(fz_context *ctx, fz_stext_device *dev, fz_text_span *span, cons
 		else
 			adv = 0;
 
-		fz_add_stext_char(ctx, dev, font, span->items[i].ucs, span->items[i].gid, &trm, adv, span->wmode);
+		fz_add_stext_char(ctx, dev, font, span->items[i].ucs, span->items[i].gid, trm, adv, span->wmode);
 	}
 }
 
 static void
-fz_stext_fill_text(fz_context *ctx, fz_device *dev, const fz_text *text, const fz_matrix *ctm,
+fz_stext_fill_text(fz_context *ctx, fz_device *dev, const fz_text *text, fz_matrix ctm,
 	fz_colorspace *colorspace, const float *color, float alpha, const fz_color_params *color_params)
 {
 	fz_stext_device *tdev = (fz_stext_device*)dev;
@@ -520,7 +505,7 @@ fz_stext_fill_text(fz_context *ctx, fz_device *dev, const fz_text *text, const f
 }
 
 static void
-fz_stext_stroke_text(fz_context *ctx, fz_device *dev, const fz_text *text, const fz_stroke_state *stroke, const fz_matrix *ctm,
+fz_stext_stroke_text(fz_context *ctx, fz_device *dev, const fz_text *text, const fz_stroke_state *stroke, fz_matrix ctm,
 	fz_colorspace *colorspace, const float *color, float alpha, const fz_color_params *color_params)
 {
 	fz_stext_device *tdev = (fz_stext_device*)dev;
@@ -531,7 +516,7 @@ fz_stext_stroke_text(fz_context *ctx, fz_device *dev, const fz_text *text, const
 }
 
 static void
-fz_stext_clip_text(fz_context *ctx, fz_device *dev, const fz_text *text, const fz_matrix *ctm, const fz_rect *scissor)
+fz_stext_clip_text(fz_context *ctx, fz_device *dev, const fz_text *text, fz_matrix ctm, fz_rect scissor)
 {
 	fz_stext_device *tdev = (fz_stext_device*)dev;
 	fz_text_span *span;
@@ -541,7 +526,7 @@ fz_stext_clip_text(fz_context *ctx, fz_device *dev, const fz_text *text, const f
 }
 
 static void
-fz_stext_clip_stroke_text(fz_context *ctx, fz_device *dev, const fz_text *text, const fz_stroke_state *stroke, const fz_matrix *ctm, const fz_rect *scissor)
+fz_stext_clip_stroke_text(fz_context *ctx, fz_device *dev, const fz_text *text, const fz_stroke_state *stroke, fz_matrix ctm, fz_rect scissor)
 {
 	fz_stext_device *tdev = (fz_stext_device*)dev;
 	fz_text_span *span;
@@ -551,7 +536,7 @@ fz_stext_clip_stroke_text(fz_context *ctx, fz_device *dev, const fz_text *text,
 }
 
 static void
-fz_stext_ignore_text(fz_context *ctx, fz_device *dev, const fz_text *text, const fz_matrix *ctm)
+fz_stext_ignore_text(fz_context *ctx, fz_device *dev, const fz_text *text, fz_matrix ctm)
 {
 	fz_stext_device *tdev = (fz_stext_device*)dev;
 	fz_text_span *span;
@@ -563,7 +548,7 @@ fz_stext_ignore_text(fz_context *ctx, fz_device *dev, const fz_text *text, const
 /* Images and shadings */
 
 static void
-fz_stext_fill_image(fz_context *ctx, fz_device *dev, fz_image *img, const fz_matrix *ctm, float alpha, const fz_color_params *color_params)
+fz_stext_fill_image(fz_context *ctx, fz_device *dev, fz_image *img, fz_matrix ctm, float alpha, const fz_color_params *color_params)
 {
 	fz_stext_device *tdev = (fz_stext_device*)dev;
 
@@ -575,14 +560,14 @@ fz_stext_fill_image(fz_context *ctx, fz_device *dev, fz_image *img, const fz_mat
 }
 
 static void
-fz_stext_fill_image_mask(fz_context *ctx, fz_device *dev, fz_image *img, const fz_matrix *ctm,
+fz_stext_fill_image_mask(fz_context *ctx, fz_device *dev, fz_image *img, fz_matrix ctm,
 		fz_colorspace *cspace, const float *color, float alpha, const fz_color_params *color_params)
 {
 	fz_stext_fill_image(ctx, dev, img, ctm, alpha, color_params);
 }
 
 static fz_image *
-fz_new_image_from_shade(fz_context *ctx, fz_shade *shade, fz_matrix *in_out_ctm, const fz_color_params *color_params, const fz_rect *scissor)
+fz_new_image_from_shade(fz_context *ctx, fz_shade *shade, fz_matrix *in_out_ctm, const fz_color_params *color_params, fz_rect scissor)
 {
 	fz_matrix ctm = *in_out_ctm;
 	fz_pixmap *pix;
@@ -590,18 +575,18 @@ fz_new_image_from_shade(fz_context *ctx, fz_shade *shade, fz_matrix *in_out_ctm,
 	fz_rect bounds;
 	fz_irect bbox;
 
-	fz_bound_shade(ctx, shade, &ctm, &bounds);
-	fz_intersect_rect(&bounds, scissor);
-	fz_irect_from_rect(&bbox, &bounds);
+	bounds = fz_bound_shade(ctx, shade, ctm);
+	bounds = fz_intersect_rect(bounds, scissor);
+	bbox = fz_irect_from_rect(bounds);
 
-	pix = fz_new_pixmap_with_bbox(ctx, fz_device_rgb(ctx), &bbox, NULL, !shade->use_background);
+	pix = fz_new_pixmap_with_bbox(ctx, fz_device_rgb(ctx), bbox, NULL, !shade->use_background);
 	fz_try(ctx)
 	{
 		if (shade->use_background)
 			fz_fill_pixmap_with_color(ctx, pix, shade->colorspace, shade->background, color_params);
 		else
 			fz_clear_pixmap(ctx, pix);
-		fz_paint_shade(ctx, shade, NULL, &ctm, pix, color_params, &bbox, NULL);
+		fz_paint_shade(ctx, shade, NULL, ctm, pix, color_params, bbox, NULL);
 		img = fz_new_image_from_pixmap(ctx, pix, NULL);
 	}
 	fz_always(ctx)
@@ -619,13 +604,13 @@ fz_new_image_from_shade(fz_context *ctx, fz_shade *shade, fz_matrix *in_out_ctm,
 }
 
 static void
-fz_stext_fill_shade(fz_context *ctx, fz_device *dev, fz_shade *shade, const fz_matrix *ctm, float alpha, const fz_color_params *color_params)
+fz_stext_fill_shade(fz_context *ctx, fz_device *dev, fz_shade *shade, fz_matrix ctm, float alpha, const fz_color_params *color_params)
 {
-	fz_matrix local_ctm = *ctm;
-	const fz_rect *scissor = fz_device_current_scissor(ctx, dev);
+	fz_matrix local_ctm = ctm;
+	fz_rect scissor = fz_device_current_scissor(ctx, dev);
 	fz_image *image = fz_new_image_from_shade(ctx, shade, &local_ctm, color_params, scissor);
 	fz_try(ctx)
-		fz_stext_fill_image(ctx, dev, image, &local_ctm, alpha, color_params);
+		fz_stext_fill_image(ctx, dev, image, local_ctm, alpha, color_params);
 	fz_always(ctx)
 		fz_drop_image(ctx, image);
 	fz_catch(ctx)
@@ -648,8 +633,8 @@ fz_stext_close_device(fz_context *ctx, fz_device *dev)
 		for (line = block->u.t.first_line; line; line = line->next)
 		{
 			for (ch = line->first_char; ch; ch = ch->next)
-				fz_union_rect(&line->bbox, &ch->bbox);
-			fz_union_rect(&block->bbox, &line->bbox);
+				line->bbox = fz_union_rect(line->bbox, fz_rect_from_quad(ch->quad));
+			block->bbox = fz_union_rect(block->bbox, line->bbox);
 		}
 	}
 
