@@ -4,59 +4,6 @@
 #include <limits.h>
 #include <assert.h>
 
-int fz_stext_char_count(fz_context *ctx, fz_stext_page *page)
-{
-	fz_stext_block *block;
-	fz_stext_line *line;
-	fz_stext_char *ch;
-	int len = 0;
-
-	for (block = page->first_block; block; block = block->next)
-	{
-		if (block->type != FZ_STEXT_BLOCK_TEXT)
-			continue;
-		for (line = block->u.t.first_line; line; line = line->next)
-		{
-			for (ch = line->first_char; ch; ch = ch->next)
-				++len;
-			++len; /* pseudo-newline */
-		}
-	}
-
-	return len;
-}
-
-const fz_stext_char *fz_stext_char_at(fz_context *ctx, fz_stext_page *page, int idx)
-{
-	static const fz_stext_char space = { ' ', {0,0}, {0,0,0,0}, 0, NULL, NULL };
-	static const fz_stext_char zero = { '\0', {0,0}, {0,0,0,0}, 0, NULL, NULL };
-	fz_stext_block *block;
-	fz_stext_line *line;
-	fz_stext_char *ch;
-	int ofs = 0;
-
-	for (block = page->first_block; block; block = block->next)
-	{
-		if (block->type != FZ_STEXT_BLOCK_TEXT)
-			continue;
-		for (line = block->u.t.first_line; line; line = line->next)
-		{
-			for (ch = line->first_char; ch; ch = ch->next)
-			{
-				if (ofs == idx)
-					return ch;
-				++ofs;
-			}
-
-			/* pseudo-newline */
-			if (idx == ofs)
-				return &space;
-			++ofs;
-		}
-	}
-	return &zero;
-}
-
 /* Enumerate marked selection */
 
 static float dist2(float a, float b)
@@ -64,6 +11,20 @@ static float dist2(float a, float b)
 	return a * a + b * b;
 }
 
+static float hdist(fz_point *dir, fz_point *a, fz_point *b)
+{
+	float dx = b->x - a->x;
+	float dy = b->y - a->y;
+	return fz_abs(dx * dir->x + dy * dir->y);
+}
+
+static float vdist(fz_point *dir, fz_point *a, fz_point *b)
+{
+	float dx = b->x - a->x;
+	float dy = b->y - a->y;
+	return fz_abs(dx * dir->y + dy * dir->x);
+}
+
 static int line_length(fz_stext_line *line)
 {
 	fz_stext_char *ch;
@@ -96,8 +57,8 @@ static int find_closest_in_line(fz_stext_line *line, int idx, fz_point p)
 
 	for (ch = line->first_char; ch; ch = ch->next)
 	{
-		float mid_x = (ch->bbox.x0 + ch->bbox.x1) / 2;
-		float mid_y = (ch->bbox.y0 + ch->bbox.y1) / 2;
+		float mid_x = (ch->quad.ul.x + ch->quad.ur.x + ch->quad.ll.x + ch->quad.lr.x) / 4;
+		float mid_y = (ch->quad.ul.y + ch->quad.ur.y + ch->quad.ll.y + ch->quad.lr.y) / 4;
 		float this_dist = dist2(p.x - mid_x, p.y - mid_y);
 		if (this_dist < closest_dist)
 		{
@@ -219,12 +180,79 @@ fz_enumerate_selection(fz_context *ctx, fz_stext_page *page, fz_point a, fz_poin
 	}
 }
 
+fz_quad
+fz_snap_selection(fz_context *ctx, fz_stext_page *page, fz_point *a, fz_point *b, int mode)
+{
+	fz_stext_block *block;
+	fz_stext_line *line;
+	fz_stext_char *ch;
+	fz_quad handles;
+	int idx, start, end;
+	int pc;
+
+	start = find_closest_in_page(page, *a);
+	end = find_closest_in_page(page, *b);
+
+	if (start > end)
+		idx = start, start = end, end = idx;
+
+	handles.ll = handles.ul = *a;
+	handles.lr = handles.ur = *b;
+
+	idx = 0;
+	for (block = page->first_block; block; block = block->next)
+	{
+		if (block->type != FZ_STEXT_BLOCK_TEXT)
+			continue;
+		for (line = block->u.t.first_line; line; line = line->next)
+		{
+			pc = '\n';
+			for (ch = line->first_char; ch; ch = ch->next)
+			{
+				if (idx <= start)
+				{
+					if (mode == FZ_SELECT_CHARS
+						|| (mode == FZ_SELECT_WORDS && (pc == ' ' || pc == '\n'))
+						|| (mode == FZ_SELECT_LINES && (pc == '\n')))
+					{
+						handles.ll = ch->quad.ll;
+						handles.ul = ch->quad.ul;
+						*a = ch->origin;
+					}
+				}
+				if (idx >= end)
+				{
+					if (mode == FZ_SELECT_CHARS
+						|| (mode == FZ_SELECT_WORDS && (ch->c == ' ')))
+					{
+						handles.lr = ch->quad.ll;
+						handles.ur = ch->quad.ul;
+						*b = ch->origin;
+						return handles;
+					}
+					if (!ch->next)
+					{
+						handles.lr = ch->quad.lr;
+						handles.ur = ch->quad.ur;
+						*b = ch->quad.lr;
+						return handles;
+					}
+				}
+				pc = ch->c;
+				++idx;
+			}
+		}
+	}
+
+	return handles;
+}
+
 /* Highlight selection */
 
 struct highlight
 {
 	int len, cap;
-	fz_rect *box;
+	fz_quad *box;
 	float hfuzz, vfuzz;
 };
 
@@ -233,80 +261,23 @@ static void on_highlight_char(fz_context *ctx, void *arg, fz_stext_line *line, f
 	struct highlight *hits = arg;
 	float vfuzz = ch->size * hits->vfuzz;
 	float hfuzz = ch->size * hits->hfuzz;
-	fz_rect bbox;
-
-	if (line->dir.x > line->dir.y)
-	{
-		bbox.x0 = ch->bbox.x0;
-		bbox.x1 = ch->bbox.x1;
-		bbox.y0 = line->bbox.y0;
-		bbox.y1 = line->bbox.y1;
-	}
-	else
-	{
-		bbox.x0 = line->bbox.x0;
-		bbox.x1 = line->bbox.x1;
-		bbox.y0 = ch->bbox.y0;
-		bbox.y1 = ch->bbox.y1;
-	}
 
 	if (hits->len > 0)
 	{
-		fz_rect *end = &hits->box[hits->len-1];
-		if (fz_abs(bbox.y0 - end->y0) < vfuzz && fz_abs(bbox.y1 - end->y1) < vfuzz)
+		fz_quad *end = &hits->box[hits->len-1];
+		if (hdist(&line->dir, &end->lr, &ch->quad.ll) < hfuzz
+			&& vdist(&line->dir, &end->lr, &ch->quad.ll) < vfuzz
+			&& hdist(&line->dir, &end->ur, &ch->quad.ul) < hfuzz
+			&& vdist(&line->dir, &end->ur, &ch->quad.ul) < vfuzz)
 		{
-			if (bbox.x1 < end->x0)
-			{
-				if (end->x0 - bbox.x1 < hfuzz)
-				{
-					end->x0 = bbox.x0;
-					return;
-				}
-			}
-			else if (bbox.x0 > end->x1)
-			{
-				if (bbox.x0 - end->x1 < hfuzz)
-				{
-					end->x1 = bbox.x1;
-					return;
-				}
-			}
-			else
-			{
-				end->x0 = fz_min(bbox.x0, end->x0);
-				end->x1 = fz_max(bbox.x1, end->x1);
-				return;
-			}
-		}
-		if (fz_abs(bbox.x0 - end->x0) < vfuzz && fz_abs(bbox.x1 - end->x1) < vfuzz)
-		{
-			if (bbox.y1 < end->y0)
-			{
-				if (end->y0 - bbox.y1 < hfuzz)
-				{
-					end->y0 = bbox.y0;
-					return;
-				}
-			}
-			else if (bbox.y0 > end->y1)
-			{
-				if (bbox.y0 - end->y1 < hfuzz)
-				{
-					end->y1 = bbox.y1;
-					return;
-				}
-			}
-			else
-			{
-				end->y0 = fz_min(bbox.y0, end->y0);
-				end->y1 = fz_max(bbox.y1, end->y1);
-				return;
-			}
+			end->ur = ch->quad.ur;
+			end->lr = ch->quad.lr;
+			return;
 		}
 	}
 
 	if (hits->len < hits->cap)
-		hits->box[hits->len++] = bbox;
+		hits->box[hits->len++] = ch->quad;
 }
 
 static void on_highlight_line(fz_context *ctx, void *arg, fz_stext_line *line)
@@ -314,14 +285,14 @@ static void on_highlight_line(fz_context *ctx, void *arg, fz_stext_line *line)
 }
 
 int
-fz_highlight_selection(fz_context *ctx, fz_stext_page *page, fz_point a, fz_point b, fz_rect *hit_bbox, int hit_max)
+fz_highlight_selection(fz_context *ctx, fz_stext_page *page, fz_point a, fz_point b, fz_quad *quads, int max_quads)
 {
 	struct callbacks cb;
 	struct highlight hits;
 
 	hits.len = 0;
-	hits.cap = hit_max;
-	hits.box = hit_bbox;
+	hits.cap = max_quads;
+	hits.box = quads;
 	hits.hfuzz = 0.5f;
 	hits.vfuzz = 0.1f;
 
@@ -440,7 +411,7 @@ static const char *find_string(const char *s, const char *needle, const char **e
 }
 
 int
-fz_search_stext_page(fz_context *ctx, fz_stext_page *page, const char *needle, fz_rect *hit_bbox, int hit_max)
+fz_search_stext_page(fz_context *ctx, fz_stext_page *page, const char *needle, fz_quad *quads, int max_quads)
 {
 	struct highlight hits;
 	fz_stext_block *block;
@@ -454,8 +425,8 @@ fz_search_stext_page(fz_context *ctx, fz_stext_page *page, const char *needle, f
 		return 0;
 
 	hits.len = 0;
-	hits.cap = hit_max;
-	hits.box = hit_bbox;
+	hits.cap = max_quads;
+	hits.box = quads;
 	hits.hfuzz = 0.1f;
 	hits.vfuzz = 0.1f;
 
