@@ -55,13 +55,13 @@ pdf_load_image_imp(fz_context *ctx, pdf_document *doc, pdf_obj *rdb, pdf_obj *di
 	if (pdf_is_jpx_image(ctx, dict))
 		return pdf_load_jpx_imp(ctx, doc, rdb, dict, cstm, forcemask);
 
-	w = pdf_to_int(ctx, pdf_dict_geta(ctx, dict, PDF_NAME_Width, PDF_NAME_W));
-	h = pdf_to_int(ctx, pdf_dict_geta(ctx, dict, PDF_NAME_Height, PDF_NAME_H));
-	bpc = pdf_to_int(ctx, pdf_dict_geta(ctx, dict, PDF_NAME_BitsPerComponent, PDF_NAME_BPC));
+	w = pdf_to_int(ctx, pdf_dict_geta(ctx, dict, PDF_NAME(Width), PDF_NAME(W)));
+	h = pdf_to_int(ctx, pdf_dict_geta(ctx, dict, PDF_NAME(Height), PDF_NAME(H)));
+	bpc = pdf_to_int(ctx, pdf_dict_geta(ctx, dict, PDF_NAME(BitsPerComponent), PDF_NAME(BPC)));
 	if (bpc == 0)
 		bpc = 8;
-	imagemask = pdf_to_bool(ctx, pdf_dict_geta(ctx, dict, PDF_NAME_ImageMask, PDF_NAME_IM));
-	interpolate = pdf_to_bool(ctx, pdf_dict_geta(ctx, dict, PDF_NAME_Interpolate, PDF_NAME_I));
+	imagemask = pdf_to_bool(ctx, pdf_dict_geta(ctx, dict, PDF_NAME(ImageMask), PDF_NAME(IM)));
+	interpolate = pdf_to_bool(ctx, pdf_dict_geta(ctx, dict, PDF_NAME(Interpolate), PDF_NAME(I)));
 
 	indexed = 0;
 	use_colorkey = 0;
@@ -88,13 +88,13 @@ pdf_load_image_imp(fz_context *ctx, pdf_document *doc, pdf_obj *rdb, pdf_obj *di
 
 	fz_try(ctx)
 	{
-		obj = pdf_dict_geta(ctx, dict, PDF_NAME_ColorSpace, PDF_NAME_CS);
+		obj = pdf_dict_geta(ctx, dict, PDF_NAME(ColorSpace), PDF_NAME(CS));
 		if (obj && !imagemask && !forcemask)
 		{
 			/* colorspace resource lookup is only done for inline images */
 			if (pdf_is_name(ctx, obj))
 			{
-				res = pdf_dict_get(ctx, pdf_dict_get(ctx, rdb, PDF_NAME_ColorSpace), obj);
+				res = pdf_dict_get(ctx, pdf_dict_get(ctx, rdb, PDF_NAME(ColorSpace)), obj);
 				if (res)
 					obj = res;
 			}
@@ -109,11 +109,11 @@ pdf_load_image_imp(fz_context *ctx, pdf_document *doc, pdf_obj *rdb, pdf_obj *di
 			n = 1;
 		}
 
-		obj = pdf_dict_geta(ctx, dict, PDF_NAME_Decode, PDF_NAME_D);
+		obj = pdf_dict_geta(ctx, dict, PDF_NAME(Decode), PDF_NAME(D));
 		if (obj)
 		{
 			for (i = 0; i < n * 2; i++)
-				decode[i] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i));
+				decode[i] = pdf_array_get_real(ctx, obj, i);
 		}
 		else if (fz_colorspace_is_lab(ctx, colorspace) || fz_colorspace_is_lab_icc(ctx, colorspace))
 		{
@@ -131,7 +131,7 @@ pdf_load_image_imp(fz_context *ctx, pdf_document *doc, pdf_obj *rdb, pdf_obj *di
 				decode[i] = i & 1 ? maxval : 0;
 		}
 
-		obj = pdf_dict_geta(ctx, dict, PDF_NAME_SMask, PDF_NAME_Mask);
+		obj = pdf_dict_geta(ctx, dict, PDF_NAME(SMask), PDF_NAME(Mask));
 		if (pdf_is_dict(ctx, obj))
 		{
 			/* Not allowed for inline images or soft masks */
@@ -142,12 +142,12 @@ pdf_load_image_imp(fz_context *ctx, pdf_document *doc, pdf_obj *rdb, pdf_obj *di
 			else
 			{
 				mask = pdf_load_image_imp(ctx, doc, rdb, obj, NULL, 1);
-				obj = pdf_dict_get(ctx, obj, PDF_NAME_Matte);
+				obj = pdf_dict_get(ctx, obj, PDF_NAME(Matte));
 				if (pdf_is_array(ctx, obj))
 				{
 					use_colorkey = 1;
 					for (i = 0; i < n; i++)
-						colorkey[i] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i)) * 255;
+						colorkey[i] = pdf_array_get_real(ctx, obj, i) * 255;
 				}
 			}
 		}
@@ -161,7 +161,7 @@ pdf_load_image_imp(fz_context *ctx, pdf_document *doc, pdf_obj *rdb, pdf_obj *di
 					fz_warn(ctx, "invalid value in color key mask");
 					use_colorkey = 0;
 				}
-				colorkey[i] = pdf_to_int(ctx, pdf_array_get(ctx, obj, i));
+				colorkey[i] = pdf_array_get_int(ctx, obj, i);
 			}
 		}
 
@@ -207,12 +207,12 @@ pdf_is_jpx_image(fz_context *ctx, pdf_obj *dict)
 	pdf_obj *filter;
 	int i, n;
 
-	filter = pdf_dict_get(ctx, dict, PDF_NAME_Filter);
-	if (pdf_name_eq(ctx, filter, PDF_NAME_JPXDecode))
+	filter = pdf_dict_get(ctx, dict, PDF_NAME(Filter));
+	if (pdf_name_eq(ctx, filter, PDF_NAME(JPXDecode)))
 		return 1;
 	n = pdf_array_len(ctx, filter);
 	for (i = 0; i < n; i++)
-		if (pdf_name_eq(ctx, pdf_array_get(ctx, filter, i), PDF_NAME_JPXDecode))
+		if (pdf_name_eq(ctx, pdf_array_get(ctx, filter, i), PDF_NAME(JPXDecode)))
 			return 1;
 	return 0;
 }
@@ -240,14 +240,14 @@ pdf_load_jpx(fz_context *ctx, pdf_document *doc, pdf_obj *dict, int forcemask)
 		unsigned char *data;
 		size_t len;
 
-		obj = pdf_dict_get(ctx, dict, PDF_NAME_ColorSpace);
+		obj = pdf_dict_get(ctx, dict, PDF_NAME(ColorSpace));
 		if (obj)
 			colorspace = pdf_load_colorspace(ctx, obj);
 
 		len = fz_buffer_storage(ctx, buf, &data);
 		pix = fz_load_jpx(ctx, data, len, colorspace);
 
-		obj = pdf_dict_geta(ctx, dict, PDF_NAME_SMask, PDF_NAME_Mask);
+		obj = pdf_dict_geta(ctx, dict, PDF_NAME(SMask), PDF_NAME(Mask));
 		if (pdf_is_dict(ctx, obj))
 		{
 			if (forcemask)
@@ -256,14 +256,14 @@ pdf_load_jpx(fz_context *ctx, pdf_document *doc, pdf_obj *dict, int forcemask)
 				mask = pdf_load_image_imp(ctx, doc, NULL, obj, NULL, 1);
 		}
 
-		obj = pdf_dict_geta(ctx, dict, PDF_NAME_Decode, PDF_NAME_D);
+		obj = pdf_dict_geta(ctx, dict, PDF_NAME(Decode), PDF_NAME(D));
 		if (obj && !fz_colorspace_is_indexed(ctx, colorspace))
 		{
 			float decode[FZ_MAX_COLORS * 2];
 			int i;
 
 			for (i = 0; i < pix->n * 2; i++)
-				decode[i] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i));
+				decode[i] = pdf_array_get_real(ctx, obj, i);
 
 			fz_decode_tile(ctx, pix, decode);
 		}
@@ -323,12 +323,12 @@ pdf_add_image(fz_context *ctx, pdf_document *doc, fz_image *image, int mask)
 	if (imref)
 		return imref;
 
+	imobj = pdf_add_new_dict(ctx, doc, 3);
 	fz_try(ctx)
 	{
-		imobj = pdf_new_dict(ctx, doc, 3);
-		pdf_dict_put_drop(ctx, imobj, PDF_NAME_DecodeParms, dp = pdf_new_dict(ctx, doc, 3));
-		pdf_dict_put_drop(ctx, imobj, PDF_NAME_Type, PDF_NAME_XObject);
-		pdf_dict_put_drop(ctx, imobj, PDF_NAME_Subtype, PDF_NAME_Image);
+		dp = pdf_dict_put_dict(ctx, imobj, PDF_NAME(DecodeParms), 3);
+		pdf_dict_put(ctx, imobj, PDF_NAME(Type), PDF_NAME(XObject));
+		pdf_dict_put(ctx, imobj, PDF_NAME(Subtype), PDF_NAME(Image));
 
 		if (cbuffer)
 		{
@@ -339,65 +339,65 @@ pdf_add_image(fz_context *ctx, pdf_document *doc, fz_image *image, int mask)
 				goto raw_or_unknown_compression;
 			case FZ_IMAGE_JPEG:
 				if (cp->u.jpeg.color_transform != -1)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_ColorTransform, pdf_new_int(ctx, doc, cp->u.jpeg.color_transform));
-				pdf_dict_put_drop(ctx, imobj, PDF_NAME_Filter, PDF_NAME_DCTDecode);
+					pdf_dict_put_int(ctx, dp, PDF_NAME(ColorTransform), cp->u.jpeg.color_transform);
+				pdf_dict_put(ctx, imobj, PDF_NAME(Filter), PDF_NAME(DCTDecode));
 				break;
 			case FZ_IMAGE_JPX:
 				if (cp->u.jpx.smask_in_data)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_SMaskInData, pdf_new_int(ctx, doc, cp->u.jpx.smask_in_data));
-				pdf_dict_put_drop(ctx, imobj, PDF_NAME_Filter, PDF_NAME_JPXDecode);
+					pdf_dict_put_int(ctx, dp, PDF_NAME(SMaskInData), cp->u.jpx.smask_in_data);
+				pdf_dict_put(ctx, imobj, PDF_NAME(Filter), PDF_NAME(JPXDecode));
 				break;
 			case FZ_IMAGE_FAX:
 				if (cp->u.fax.columns)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_Columns, pdf_new_int(ctx, doc, cp->u.fax.columns));
+					pdf_dict_put_int(ctx, dp, PDF_NAME(Columns), cp->u.fax.columns);
 				if (cp->u.fax.rows)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_Rows, pdf_new_int(ctx, doc, cp->u.fax.rows));
+					pdf_dict_put_int(ctx, dp, PDF_NAME(Rows), cp->u.fax.rows);
 				if (cp->u.fax.k)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_K, pdf_new_int(ctx, doc, cp->u.fax.k));
+					pdf_dict_put_int(ctx, dp, PDF_NAME(K), cp->u.fax.k);
 				if (cp->u.fax.end_of_line)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_EndOfLine, pdf_new_int(ctx, doc, cp->u.fax.end_of_line));
+					pdf_dict_put_bool(ctx, dp, PDF_NAME(EndOfLine), cp->u.fax.end_of_line);
 				if (cp->u.fax.encoded_byte_align)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_EncodedByteAlign, pdf_new_int(ctx, doc, cp->u.fax.encoded_byte_align));
+					pdf_dict_put_bool(ctx, dp, PDF_NAME(EncodedByteAlign), cp->u.fax.encoded_byte_align);
 				if (cp->u.fax.end_of_block)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_EndOfBlock, pdf_new_int(ctx, doc, cp->u.fax.end_of_block));
+					pdf_dict_put_bool(ctx, dp, PDF_NAME(EndOfBlock), cp->u.fax.end_of_block);
 				if (cp->u.fax.black_is_1)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_BlackIs1, pdf_new_int(ctx, doc, cp->u.fax.black_is_1));
+					pdf_dict_put_bool(ctx, dp, PDF_NAME(BlackIs1), cp->u.fax.black_is_1);
 				if (cp->u.fax.damaged_rows_before_error)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_DamagedRowsBeforeError, pdf_new_int(ctx, doc, cp->u.fax.damaged_rows_before_error));
-				pdf_dict_put_drop(ctx, imobj, PDF_NAME_Filter, PDF_NAME_CCITTFaxDecode);
+					pdf_dict_put_int(ctx, dp, PDF_NAME(DamagedRowsBeforeError), cp->u.fax.damaged_rows_before_error);
+				pdf_dict_put(ctx, imobj, PDF_NAME(Filter), PDF_NAME(CCITTFaxDecode));
 				break;
 			case FZ_IMAGE_FLATE:
 				if (cp->u.flate.columns)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_Columns, pdf_new_int(ctx, doc, cp->u.flate.columns));
+					pdf_dict_put_int(ctx, dp, PDF_NAME(Columns), cp->u.flate.columns);
 				if (cp->u.flate.colors)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_Colors, pdf_new_int(ctx, doc, cp->u.flate.colors));
+					pdf_dict_put_int(ctx, dp, PDF_NAME(Colors), cp->u.flate.colors);
 				if (cp->u.flate.predictor)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_Predictor, pdf_new_int(ctx, doc, cp->u.flate.predictor));
+					pdf_dict_put_int(ctx, dp, PDF_NAME(Predictor), cp->u.flate.predictor);
 				if (cp->u.flate.bpc)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_BitsPerComponent, pdf_new_int(ctx, doc, cp->u.flate.bpc));
-				pdf_dict_put_drop(ctx, imobj, PDF_NAME_Filter, PDF_NAME_FlateDecode);
-				pdf_dict_put_drop(ctx, imobj, PDF_NAME_BitsPerComponent, pdf_new_int(ctx, doc, image->bpc));
+					pdf_dict_put_int(ctx, dp, PDF_NAME(BitsPerComponent), cp->u.flate.bpc);
+				pdf_dict_put(ctx, imobj, PDF_NAME(Filter), PDF_NAME(FlateDecode));
+				pdf_dict_put_int(ctx, imobj, PDF_NAME(BitsPerComponent), image->bpc);
 				break;
 			case FZ_IMAGE_LZW:
 				if (cp->u.lzw.columns)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_Columns, pdf_new_int(ctx, doc, cp->u.lzw.columns));
+					pdf_dict_put_int(ctx, dp, PDF_NAME(Columns), cp->u.lzw.columns);
 				if (cp->u.lzw.colors)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_Colors, pdf_new_int(ctx, doc, cp->u.lzw.colors));
+					pdf_dict_put_int(ctx, dp, PDF_NAME(Colors), cp->u.lzw.colors);
 				if (cp->u.lzw.predictor)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_Predictor, pdf_new_int(ctx, doc, cp->u.lzw.predictor));
+					pdf_dict_put_int(ctx, dp, PDF_NAME(Predictor), cp->u.lzw.predictor);
 				if (cp->u.lzw.early_change)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_EarlyChange, pdf_new_int(ctx, doc, cp->u.lzw.early_change));
+					pdf_dict_put_int(ctx, dp, PDF_NAME(EarlyChange), cp->u.lzw.early_change);
 				if (cp->u.lzw.bpc)
-					pdf_dict_put_drop(ctx, dp, PDF_NAME_BitsPerComponent, pdf_new_int(ctx, doc, cp->u.lzw.bpc));
-				pdf_dict_put_drop(ctx, imobj, PDF_NAME_Filter, PDF_NAME_LZWDecode);
+					pdf_dict_put_int(ctx, dp, PDF_NAME(BitsPerComponent), cp->u.lzw.bpc);
+				pdf_dict_put(ctx, imobj, PDF_NAME(Filter), PDF_NAME(LZWDecode));
 				break;
 			case FZ_IMAGE_RLD:
-				pdf_dict_put_drop(ctx, imobj, PDF_NAME_Filter, PDF_NAME_RunLengthDecode);
+				pdf_dict_put(ctx, imobj, PDF_NAME(Filter), PDF_NAME(RunLengthDecode));
 				break;
 			}
 
 			if (!pdf_dict_len(ctx, dp))
-				pdf_dict_del(ctx, imobj, PDF_NAME_DecodeParms);
+				pdf_dict_del(ctx, imobj, PDF_NAME(DecodeParms));
 
 			buffer = fz_keep_buffer(ctx, cbuffer->buffer);
 		}
@@ -454,73 +454,83 @@ raw_or_unknown_compression:
 			}
 		}
 
-		pdf_dict_put_drop(ctx, imobj, PDF_NAME_Width, pdf_new_int(ctx, doc, pixmap ? pixmap->w : image->w));
-		pdf_dict_put_drop(ctx, imobj, PDF_NAME_Height, pdf_new_int(ctx, doc, pixmap ? pixmap->h : image->h));
+		pdf_dict_put_int(ctx, imobj, PDF_NAME(Width), pixmap ? pixmap->w : image->w);
+		pdf_dict_put_int(ctx, imobj, PDF_NAME(Height), pixmap ? pixmap->h : image->h);
 
 		if (mask)
 		{
-			pdf_dict_put_drop(ctx, imobj, PDF_NAME_ImageMask, pdf_new_bool(ctx, doc, 1));
+			pdf_dict_put_bool(ctx, imobj, PDF_NAME(ImageMask), 1);
 		}
 		else
 		{
 			fz_colorspace *cs;
 
-			pdf_dict_put_drop(ctx, imobj, PDF_NAME_BitsPerComponent, pdf_new_int(ctx, doc, image->bpc));
+			pdf_dict_put_int(ctx, imobj, PDF_NAME(BitsPerComponent), image->bpc);
 
 			cs = pixmap ? pixmap->colorspace : image->colorspace;
-			n = fz_colorspace_n(ctx, cs);
-
-			if (fz_colorspace_is_indexed(ctx, cs))
+			switch (fz_colorspace_type(ctx, cs))
 			{
-				fz_colorspace *basecs;
-				unsigned char *lookup = NULL;
-				int high = 0;
-				int basen;
-				pdf_obj *arr;
+			case FZ_COLORSPACE_INDEXED:
+				{
+					fz_colorspace *basecs;
+					unsigned char *lookup = NULL;
+					int high = 0;
+					int basen;
+					pdf_obj *arr;
 
-				lookup = fz_indexed_colorspace_palette(ctx, cs, &high);
-				basecs = fz_colorspace_base(ctx, cs);
-				basen = fz_colorspace_n(ctx, basecs);
+					lookup = fz_indexed_colorspace_palette(ctx, cs, &high);
+					basecs = fz_colorspace_base(ctx, cs);
+					basen = fz_colorspace_n(ctx, basecs);
 
-				pdf_dict_put_drop(ctx, imobj, PDF_NAME_ColorSpace, arr = pdf_new_array(ctx, doc, 4));
+					arr = pdf_dict_put_array(ctx, imobj, PDF_NAME(ColorSpace), 4);
 
-				pdf_array_put_drop(ctx, arr, 0, PDF_NAME_Indexed);
-				if (basen <= 1)
-					pdf_array_put_drop(ctx, arr, 1, PDF_NAME_DeviceGray);
-				else if (basen == 3)
-					// TODO: Lab colorspace?
-					pdf_array_put_drop(ctx, arr, 1, PDF_NAME_DeviceRGB);
-				else if (basen == 4)
-					pdf_array_put_drop(ctx, arr, 1, PDF_NAME_DeviceCMYK);
-				else
-					// TODO: convert to RGB!
-					fz_throw(ctx, FZ_ERROR_GENERIC, "only indexed Gray, RGB, and CMYK colorspaces supported");
+					pdf_array_push(ctx, arr, PDF_NAME(Indexed));
+					switch (fz_colorspace_type(ctx, basecs))
+					{
+					case FZ_COLORSPACE_GRAY:
+						pdf_array_push(ctx, arr, PDF_NAME(DeviceGray));
+						break;
+					case FZ_COLORSPACE_RGB:
+						pdf_array_push(ctx, arr, PDF_NAME(DeviceRGB));
+						break;
+					case FZ_COLORSPACE_CMYK:
+						pdf_array_push(ctx, arr, PDF_NAME(DeviceCMYK));
+						break;
+					default:
+						// TODO: convert to RGB!
+						fz_throw(ctx, FZ_ERROR_GENERIC, "only indexed Gray, RGB, and CMYK colorspaces supported");
+						break;
+					}
 
-				pdf_array_put_drop(ctx, arr, 2, pdf_new_int(ctx, doc, high));
-				pdf_array_put_drop(ctx, arr, 3, pdf_new_string(ctx, doc, (char *) lookup, basen * (high + 1)));
-			}
-			else if (n <= 1)
-				pdf_dict_put_drop(ctx, imobj, PDF_NAME_ColorSpace, PDF_NAME_DeviceGray);
-			else if (n == 3)
-				// TODO: Lab colorspace?
-				pdf_dict_put_drop(ctx, imobj, PDF_NAME_ColorSpace, PDF_NAME_DeviceRGB);
-			else if (n == 4)
-				pdf_dict_put_drop(ctx, imobj, PDF_NAME_ColorSpace, PDF_NAME_DeviceCMYK);
-			else
+					pdf_array_push_int(ctx, arr, high);
+					pdf_array_push_string(ctx, arr, (char *) lookup, basen * (high + 1));
+				}
+				break;
+			case FZ_COLORSPACE_GRAY:
+				pdf_dict_put(ctx, imobj, PDF_NAME(ColorSpace), PDF_NAME(DeviceGray));
+				break;
+			case FZ_COLORSPACE_RGB:
+				pdf_dict_put(ctx, imobj, PDF_NAME(ColorSpace), PDF_NAME(DeviceRGB));
+				break;
+			case FZ_COLORSPACE_CMYK:
+				pdf_dict_put(ctx, imobj, PDF_NAME(ColorSpace), PDF_NAME(DeviceCMYK));
+				break;
+			default:
 				// TODO: convert to RGB!
 				fz_throw(ctx, FZ_ERROR_GENERIC, "only Gray, RGB, and CMYK colorspaces supported");
+				break;
+			}
 		}
 
 		if (image->mask)
 		{
-			pdf_dict_put_drop(ctx, imobj, PDF_NAME_SMask, pdf_add_image(ctx, doc, image->mask, 0));
+			pdf_dict_put_drop(ctx, imobj, PDF_NAME(SMask), pdf_add_image(ctx, doc, image->mask, 0));
 		}
 
-		imref = pdf_add_object(ctx, doc, imobj);
-		pdf_update_stream(ctx, doc, imref, buffer, 1);
+		pdf_update_stream(ctx, doc, imobj, buffer, 1);
 
 		/* Add ref to our image resource hash table. */
-		imref = pdf_insert_image_resource(ctx, doc, digest, imref);
+		imref = pdf_insert_image_resource(ctx, doc, digest, imobj);
 	}
 	fz_always(ctx)
 	{
@@ -529,9 +539,6 @@ raw_or_unknown_compression:
 		pdf_drop_obj(ctx, imobj);
 	}
 	fz_catch(ctx)
-	{
-		pdf_drop_obj(ctx, imref);
 		fz_rethrow(ctx);
-	}
 	return imref;
 }
